# SyrusJS

## v1.49.2
  - Added set counters feature

```
# First time definition, once set this value can't be modified with the same instruction
define counters globals odometer=0 ignition_time=0 idle_time=0 over_speed=0 over_rpm=0 hard_brakes=0 harsh_fwd_acceleration=0

# Change/set thresholds settings
set counters globals speed_threshold=30
set counters globals begin_idle_time=2min
set counters globals rpm_threshold=700

# Change counter value 
set counters globals odometer=0
set counters globals ignition_time=0
set counters globals idle_time=0
set counters globals over_speed=0
set counters globals hard_brakes=0
set counters globals harsh_fwd_acceleration=0
set counters globals over_rpm=0
set counters globals distance=0
```
## v1.49.0
 -  Added ECU 52keys

## v1.48.1
  - Added jamming_ended signal
## v1.48.0
  - Added support for Jamming detection

    Signals
      - @modem.jamming_detected             - Triggered when a jamming condition is detected
      - @modem.jamming_persistence          - Triggered when the jamming condition persist for one minute
      - @modem.jamming_ended                - Triggered when the jamming action stops

    States
      - $modem.jamming.state                - The jamming state [ JAMMED | NOT_JAMMING ]
      - $modem.jamming_detected             - The jamming state [ true | false ]  
      - $modem.jamming_persistence          - The persistence flag state [ true | false]
      - $modem.jamming_ended                - The ended flag state [ true | false ]

```
##### Jamming Detection ####### 

define event ev_jamming group=tracking ack=seq label=jammed code=90 fieldset=tracking trigger=@modem.jamming_detected

define event ev_jampers group=tracking ack=seq label=jampers code=91 fieldset=tracking trigger=@modem.jamming_persistence

define event ev_not_jamming group=tracking ack=seq label=jammend code=92 fieldset=tracking trigger=@modem.jamming_ended

##############################
```

## v1.47.5
  - Fixed caredrive not uploading photo bug - other events
  - Added support for apex reset event
  - Added support for apex netchange event
  
    Signals
      - @system.reset                       - Triggered at every apex reset
      - @system.netchange                   - Triggered every time the network changed after a syrusjs restart

## v1.47.3
 - Fixed caredrive not uploading photo bug
 - Fixed ble negative temperature bug
 - APEX 23.03.1

## v1.47.1
 - Added support for ECU warnings 
 - Added Support for BLE Gateway mode
 - APEX 22.51.1

  Signals
    - @ecu_warning.event                      - Triggered at any ecu warning event
    - @ecu_warning.(ECU_ID).(ECU_INTERVAL)    - Triggered only when the specified interval is reached 

  ```
  ##### ECU WARNINGS #####

  # -- Signal Use the full description to assign to a signal name
  define signal fuel_warning @ecu_warning.fd6e_1-2.delta?time=20&value=100

  # -- Event Use the signal name as trigger
  define event ev_fuel_warning group=tracking fieldset=default ack=seq label=flwrng code=90 trigger=fuel_warning

  ########################
  ```

  Signals
    - @ble.event                                   - Triggered at any ble event
    - @ble.(MAC or ALIAS).temperature.change       - Triggered when a change in temperature is detected
    
  States
    - $ble.(MAC_ADDRESS or ALIAS).mac              - Device mac address
    - $ble.(MAC_ADDRESS or ALIAS).name             - Advertised name
    - $ble.(MAC_ADDRESS or ALIAS).alias            - Assigned alias
    - $ble.(MAC_ADDRESS or ALIAS).timestamp        - Scanned time
    - $ble.(MAC_ADDRESS or ALIAS).temperature      - Reported temperature value in degrees celsius
    - $ble.(MAC_ADDRESS or ALIAS).humidity         - Reported humidity value in percentage
    - $ble.(MAC_ADDRESS or ALIAS).movement         - Reported number of movements detected

```
###### BLE GATEWAY  ######

# -- Use apx-ble to add the desired BLE devices and enable continous scanning
# apx-ble add --mac=MAC_ADDRESS --alias=cooler
# apx-ble set --scan_time=60 --enabled=true

# -- Variables definitions
define variable movement
set variable movement 0

define variable temperature 
set variable temperature 0

# -- Fieldsets
define fieldset default fields=$io,$ble

# -- Signals - Used previously defined alias "cooler"
define signal sg_hightemp $ble.cooler.temperature > 20 
define signal sg_lowtemp  $ble.cooler.temperature <= 20
define signal sg_movement $ble.E7973F682A94.movement > movement

# -- Events - trigger only used on the action
define event ev_hightemp group=tracking ack=seq label=hitemp code=70 fieldset=tracking
define event ev_lowtemp group=tracking ack=seq label=lotemp code=71 fieldset=tracking
define event ev_movement group=tracking ack=seq label=mvnt code=72 fieldset=tracking

# -- Actions
define action ac_hightemp trigger=sg_hightemp
  set variable temperature {{$ble.cooler.temperature}}
  send event ev_hightemp
  speak "High temperature detected at {{$variables.cooler.temperature}}"

define action ac_lowtemp trigger=sg_lowtemp
  set variable temperature {{$ble.cooler.temperature}}
  send event ev_lowtemp
  speak "Low temperature detected at {{$variables.cooler.temperature}}"  

define action ac_movement trigger=sg_movement
  set variable movement {{$ble.E7973F682A94.movement}}
  send event ev_movement
  speak "Movement detected at {{$variables.E7973F682A94.movement}}"  

########################
```
## v1.45.9
 - Updated ECU Keys, DAILMER.

## v1.45.6
  - Added gnss.acceleration.mphs
  - Changed taip acceleration transmitted in kphs to mph
  
## v1.45.5
  - Fixed MQTT transmission error
  - Added bluetooth status update

## v1.45.3
 - Added support for snapshots 
 ```
 ##### SNAPSHOTS CONFIG #####

# -- Video events (same event different options)
define event ev_tamper group=tracking ack=seq label=tamper code=13 fieldset=tracking
    video=$variables.braking_video
    photo=$variables.braking_video
    camera=MyCameraName
    upload=automatic

# -- Video recording action @dms.cipia.tamper_alert
define action ac_braking_video trigger=sg_test
    set variable braking_video {{$gnss.timestamp}}-braking
    create video --name={{$variables.braking_video}} --time_win=-5,+5 --with_snapshot
    send event ev_tamper

 ##### SNAPSHOTS CONFIG #####
 ```
## v1.45.2
- Added support for people counter feature
- APEX 22.43.0
  Signals 
   Events (oneshot)
    - @people.adult_enter
    - @people.adult_exit
    - @people.children_enter
    - @people.children_exit
    - @people.requested
    - @people.start
    - @people.stop
    - @people.restarted
    - @people.cam_connected
    - @people.cam_disconnected
    - @people.daily_report
    - @people.weekly_report

    States
    - $people.system_epoch
    - $people.event
    - $people.adult.enter
    - $people.adult.exit
    - $people.children.enter
    - $people.children.exit
    - $people.adults_inside
    - $people.children_inside
    - $people.global_adult.enter
    - $people.global_adult.exit
    - $people.global_children.enter
    - $people.global_children.exit

```
#### PEOPLE COUNTER #####

define signal sg_pc_max $people.adults_inside >= 20

define event ev_pc_enter group=tracking fieldset=default ack=seq label=pcenter code=60 trigger=@people.adult_enter
define event ev_pc_exit group=tracking fieldset=default ack=seq label=pcexit code=61 trigger=@people.adult_exit
define event ev_pc_max group=tracking fieldset=default ack=seq label=pcmax code=62 trigger=sg_pc_max

#### END PEOPLE COUNTER #####
```
## v1.45.0
- Added Safe Engine Cut Off feature
- APEX 22.39.0

  Signals
    Events (Oneshot booleans)
      - @seco.event               Trigger when any seco event arrives
      - @seco.enabled             Trigger when the feature mode in enabled (powered ON)
      - @seco.active              Trigger when the feature is waiting for correct moment to realize the engine cut off.       

      - @seco.idle                
			- @seco.canceled            User canceled the activation
      - @seco.power_restored      Output has been deactivated or the engine power restored
      - @seco.deactivated         Output has been deactivated or the engine power restored
			- @seco.power_suspended     Output has been activated or the engine power suspended
      - @seco.activated           Output has been activated or the engine power suspended
			- @seco.activating          Waiting for conditions to suspend the engine power (speed < 10kph for 20s)  
      - @seco.waiting_no_motion   Waiting for conditions to suspend the engine power (no motion for 5 min)
      - @seco.waiting_for_gps     Safe conditions not met, either motion was detected or vehicle speed is above 10kph
      - @seco.periodic_report     Periodic report, the SECO mechanism is running but waiting for safety conditions

    States
      - $seco.enabled             Safe engine cut off module enabled 
      - $seco.active              Safe engine cut off feature actived
      - $seco.state               Number of the current state (see apx-seco help)

    Alias
      - enable seco               Enables the safe engine module
      - disable seco              Disables the safe engine module
      - start seco                Start safe engine cut off, waiting for conditions for tunrn off motor
      - stop seco                 Stop safe engine cut off, enables the motor again   
```
#####  Safe Engine Cut Off  #####  

# (Driver state: Cutting motor)
define action test_seco_on trigger=sg_ign_on
    speak "Stopping Motor Safetly"
    exec apx-seco set --mode=enable
    exec apx-seco set --trigger=enable
    send event ev_seco_on

# (Driver state: Cutting motor)
define action test_seco_off trigger=sg_ign_off
    speak "Starting Again Motor"
    exec apx-seco set --trigger=disable
    send event ev_seco_off

# (Seco event arrived)
define action test_seco_ev trigger=@seco.event
    set variable eventNumber {{$seco.state}}
    speak "Seco event received {{$variables.eventNumber}}"

####  Safe Engine Cut Off #####

```

## v1.44.3
- Added cipia tamper_alert for ndm 0.4.4

## v1.44.1
- Fixed Ecu tires bug (single tire)

## v1.44.0
- Added Time Windows Feature
- APEX 22.38.0
  Signals
    Events (Oneshot booleans) 
    - @windows.name.start  
    - @windows.name.end         

    States
    - $windows.name.in
    - $windows.name.out

 ```syruslang example 
######   TIME WINDOWS CONFIG   ######

define window working_hours start=01/09/2022 end=30/09/2022 from=08:00:00 to=17:05:30 days=0,1,1,1,1,1,0 enabled=true

# -- Events
define event ev_start group=tracking fieldset=default ack=seq label=started code=10
define event ev_stop group=tracking fieldset=default ack=seq label=stopped code=11
define event ev_test group=tracking fieldset=default ack=seq label=test code=12

# -- Actions
# (Driver state: Working) Wroking shift started  
define action start_working trigger=@windows.working_hours.start
    speak "Working hours started"
    send event ev_start

# (Driver state: Resting) Wroking shift ended  
define action stop_working trigger=@windows.working_hours.end    
    speak "Working hours stopped"
    send event ev_stop

# (Driver state: TestButton) Test current shift time  
define action test_hours trigger=$io.in1
    speak "Testing time window {{$window.working_hours.state}}"
    send event ev_test    

######  END TIME WINDOW CONFIG   ####
``` 


## v1.43.7
- Fixed Queue and repeated tx bug
- Expanded Cipia fatigue sensor signals and events
- Requires updated cipia and ndm >= 0.4.4

  Signals
    Events (Oneshot booleans) 
    - @dms.cipia.event                    Triggered when any cipia event is detected.  
    - @dms.cipia.fatigue_warning                           
    - @dms.cipia.fatigue_alarm            
    - @dms.cipia.driver_changed
    - @dms.cipia.driver_missing
    - @dms.cipia.distraction
    - @dms.cipia.driver_identified
    - @dms.cipia.ignition_off
    - @dms.cipia.ignition_on
    - @dms.cipia.phone
    - @dms.cipia.smoking
    - @dms.cipia.seatbelt
    - @dms.cipia.seatbelt_off
    - @dms.cipia.unknown_driver
    - @dms.cipia.tamper_alert

    - @dms.cipia.system_boot
    - @dms.cipia.system_boot_failure
    - @dms.cipia.system_ok
    - @dms.cipia.system_reset 

    - @dms.cipia.system_error.event              Triggered when any error event is reported.
    - @dms.cipia.system_error.emmc_failed
    - @dms.cipia.system_error.over_exposure
    - @dms.cipia.system_error.blurred_image 
    - @dms.cipia.system_error.device_not_calibrated
    - @dms.cipia.system_error.power_lost        
    - @dms.cipia.system_error.mcu_upgrade_ok
  
    - @dms.cipia.system_error.device_shifted      
    - @dms.cipia.system_error.cable_disconected       
    - @dms.cipia.system_error.overheat      
    - @dms.cipia.system_error.gps_loss
    - @dms.cipia.system_error.sdcard_hw_malfunction    

    States                 
    - $fatigue_sensor.photos   
    - $fatigue_sensor.channel
    - $fatigue_sensor.epoch
    - $fatigue_sensor.event
    - $fatigue_sensor.media.system_epoch
    - $fatigue_sensor.media.event
    - $fatigue_sensor.media.camera
    - $fatigue_sensor.media.file.type
    - $fatigue_sensor.media.file.state
    - $fatigue_sensor.media.file.path

## v1.43.4

```syruslang example 
######   VIDEOS CONFIGURATION   ######

define event ev_wifi_connect group=tracking fieldset=default ack=seq label=wifi code=29 trigger=wifi

# -- Video events (same event different options)
define event ev_braking_video group=tracking ack=seq label=brvideo code=68 fieldset=tracking
    video=$variables.braking_video
    camera=MyCameraName
    upload=automatic

define event ev_braking_video group=tracking ack=seq label=brvideo code=68 fieldset=tracking
    video=$variables.braking_video
    camera=MyCameraName
    upload=ondemand

# -- Video recording action
define action ac_braking_video trigger=ignOn
    set variable braking_video {{$gnss.timestamp}}-braking
    create video --name={{$variables.braking_video}} --time_win=-5,+5
    send event ev_braking_video
######  END VIDEO CONFIGURATION   ####
```

## v1.43.0
- Added support for variables increments and decrements

  Signals
    States
    - $variables.name                     Stores the variable value

    Actions
    - set variable name increment         Increments the variable value by 1
    - set variable name decrement         Decrement the variable value by 1
    - set variable name reset             Set the variable value to 0

```syruslang examples
#####  START VARIABLES CONFIG #####
# -- Definition 
define variable count
set variable count 0

# -- Signal
define signal sg_in1 $io.in1
define signal count_over $variables.count >= 10

# -- Actions
define action ac_increment trigger=sg_in1 set variable count increment
define action ac_reset trigger=count_over set variable count reset

# -- Event
define event ev_count group=default fieldset=fields label=resttart code=20 trigger=count_over

#####  START VARIABLES CONFIG #####
```

## v1.42.3
- Fixed counters time and distance increments
- Fixed CSV comma separated values bug
- Fixed tracking_resolution unhandled exception

- Added parametrized path to file destination 
  - $MEDIA_PATH           Redirects to /media/mmcblk0p1
  - $USER_PATH            Redirects to /data/users/syrus4g 

## v1.42.0 
- Added support for custom counters
- Apx version 22.23.3

  Signals  

    States                 
    - $counters.name.total_count          Stores number of times a signal becomes true
    - $counters.name.total_time           Stores the total time a signal stays true, stops counting when false
    - $counters.name.time_increment       Stores the time passed while a signal is true, reset count when false
    - $counters.name.total_distance       Stores the total distance traveled while a signal is true, stops counting when false
    - $counters.name.distance_increment   Stores the distance traveled while a signal is true, reset count when false
    - $counters.name.state                Stores the counter state (true when counting)

    Actions
    - start counters name                 Start/enable the counting function and listen for change in the selected signals                  
    - stop counters name                  Stop/disable the counting function, any change in the signals will be ignored
    - reset counters name                 Reset/restart the states values, like total_count, total_time, etc ...

```syruslang examples
######   START COUNTERS CONFIGURATION   ######

# -- Definition     -  In the definition we relate any signal with the counter name, and start watching the signal (optional)
define counters name signal=sg_in1_on start=true

# -- Fieldsets
define fieldset default fields=$io,$gnss,CV01:$counters.name.total_count,CV02:$counters.name.total_time,CV03:$counters.name.total_distance

# -- Signals
#    The signal to keep the count 
define signal sg_in1_on $io.in1
define signal sg_in1_off $io.in1 == false
define signal sg_limit $counters.name.total_time >= 60

# -- ACTIONS 
#    Reset when a minute active has passed 
#define action ac_stop_ign trigger=sg_limit reset counters name


# -- Event
define event ev_count_begin group=tracking fieldset=default ack=seq label=cnt_begin code=200 trigger=sg_in1_on
define event ev_count_end group=tracking fieldset=default ack=seq label=cnt_end code=201 trigger=sg_in1_off


##### END COUNTERS CONFIGURATION ######
```
## v1.41.2

 - Added support for lazy videos
 - Fix for source, age in TAIP

## v1.41.1

### CSV support
 - Added support for CSV destinations
 - Apx version 22.25.1

```syruslang
##### START CSV LOG CONFIGURATION #####
define group blackbox

# -- Fieldsets
define fieldset csvfields fields=gnss_timestamp:$gnss.timestamp,speed:$gnss.kph,acceleration:$gnss.acceleration.value,latitude:$gnss.latitude,longitude:$gnss.longitude,heading:$gnss.heading,bearing:$gnss.bearing,altitude:$gnss.altitude,fix:$gnss.fix,sats_active:$gnss.satsActive,accuracy:$gnss.accuracy,hdop:$gnss.hdop,vdop:$gnss.vdop,pdop:$gnss.pdop,power:$io.pwr,ignition:$io.ign,input1:$io.in1

# -- Timer
define timer tm_csv duration=1sec enabled=true repeat=true

# -- Signals
define signal sg_ignON $io.ign 
define signal sg_ignOFF $io.ign == false

# -- Actions
define action ev_startlog trigger=sg_ignON start timer tm_csv
define action ev_stoplog trigger=sg_ignOFF stop timer tm_csv

# -- Events
define event ev_writelog group=blackbox fieldset=csvfields label=blackbox code=100 trigger=@timers.tm_csv,sg_ignON,and

# -- Destination
set destinations group=blackbox csvlog

# -- In destinations.syurs.conf select csv protocol ( default compress and rotate every day)
#define destination log csv file:///media/mmcblk0p1/file_csv

#####  END CSV LOG CONFIG #####
```
## v1.40.2
- Added support for gps-acceleration signals
- Works like tracking resolution triggering events whenever a predefined acceleration threshold is reached in kphs.
- Available with apex-22.25.1
- Pending for next release: Acceleration units conversion.

Signals:
- @gnss.acceleration.event     (boolean) Triggered when any gnss acceleration event detected
- @gnss.acceleration.positive  (boolean) Triggered when acceleration reaches positive threshold
- @gnss.acceleration.negative  (boolean) Triggered when acceleration reaches negative threshold

States:
- $gnss.acceleration.value     (numeric) Last acceleration value
- $gnss.acceleration.reason    (string)  Last event arrived could be (positive_acc or negative_acc)

``` Syruslang example
##### START GNSS ACCEL CONFIGURATION  #####

# -- Tracking resolution  (add optional argument)
define tracking_resolution move 30sec 35deg 10,-10
# or
define tracking_resolution name=move time=30sec heading=35deg pacc=10 nacc=-10

# -- Signals
define signal pos_acc_sg $gnss.acceleration.reason == positive_acc
define signal neg_acc_sg $gnss.acceleration.reason == negative_acc
define signal harsh_acc_sg $gnss.acceleration.value >= 10

# -- Events
define event ev_acce group=tracking fieldset=default ack=seq label=accelev code=21 trigger=@gnss.acceleration.event
define event ev_pos_acc group=tracking fieldset=default ack=seq label=accel_pos code=22 trigger=pos_acc_sg
define event ev_neg_acc group=tracking fieldset=default ack=seq label=accel_neg code=24 trigger=neg_acc_sg
define event ev_pos_harsh group=tracking fieldset=default ack=seq label=harsh code=25 trigger=harsh_acc_sg

#####   END GNSS ACCEL CONFIG  #####
```

## v1.39.1

### Backlog and Logrotate
- Added support for logrotation and backlog integration
- Apx version 22.23.3

  Signals
    Events (Oneshot booleans)
    - @logs.event                         Triggered when any log event is detected.                          

    States                 
    - $logs.name                          Stores log rotation name for apx-logrotate
    - $logs.path                          Stores log file path
    - $logs.used                          Stores media used memory
    - $logs.available                     Stores media available memory
    - $logs.size                          Stores media total size
    - $logs.percentage                    Stores media memory usage in percentage
    - $logs.period                        Stores the rotation period configuration (daily, weekly, monthly)
    - $logs.rotate                        Stores the total log files stored before rotation (ex. 5 days) 


```syruslang
######   START BACKLOGS CONFIGURATION   ######
#------- Second to second GPS black box
define group time_record

# -- Timer
define timer one_minute duration=1min enabled=true repeat=true

# -- Action
define action ac_timelog trigger=@timers.one_minute
        exec apx-gps backlog create --name=minute_backlog --time-win=-60,+0
        send event ev_ac_backlog

# -- Event
define event ev_ac_backlog group=time_record code=166 label=blog backlog=minute_backlog

# -- Destination
set destinations group=time_record taip_logs
# define destination taip_logs taip file:///media/mmcblk0p1/black_box ack=disabled rotate=5D size=10MB compress=true

# -------- Event based backlog configuration
define group break_record

# -- Signal
define signal sg_in1 min_duration=2sec $io.in1

# -- Action
define action ac_breaklog trigger=sg_in1
        exec apx-gps backlog create --name=break_backlog --time-win=-10,+10
        send event ev_button

# -- Event
define event ev_button group=break_record code=165 label=btnlog backlog=break_backlog

# -- Destination
set destinations group=break_record break_logs
# define destination break_logs taip file:///media/mmcblk0p1/break_logs ack=disabled rotate=5D size=10MB compress=true

##### END BACKLOGS CONFIGURATION ######
```

### Foton private variables
- Improved regex for ecu tags.

## v1.37.5
- Fix @tracking_resolution signals not changing.

## v1.37.4
### RFID
- Added support for serial RFID reader
- Apx version 22.17.1

  Signals
    Events (Oneshot booleans)
    - @rfid.event                         Triggered when any rfid event is detected.                          
    - @rfid.change                        Triggered when the id changed.   
    - @rfid.read                          Triggered when a card aproach the reader, even if it's the same id.  
    - @rfid.authorized                    Triggered when the id is whitelisted.   
    - @rfid.unauthorized                  Triggered when the id is not whitelisted.
    
    States                 
    - $rfid.last.id                       Stores the last id.  
    - $rfid.last.alias                    Stores the last alias.   
    - $rfid.last.whitelisted              Stores the last whitelisted state (boolean).  
    - $rfid.last.conn_epoch               Stores the last connection timestamp.   
    - $rfid.authorized.last.id            Stores the last authorized id.   
    - $rfid.authorized.last.alias         Stores the last authorized alias.   
    - $rfid.authorized.last.whitelisted   Stores the last authorized whitelisted state.   
    - $rfid.authorized.last.conn_epoch    Stores the last authorized connection timestamp.  

  ```syruslang
  # Example of event whenever a card is aproached to the reader
  define fieldset default fields=RI:$fuel.last.id
  
  define event ev_rfid group=tracking fieldset=default ack=seq label=rfid trigger=@rfid.read
  ```

  TAIP - Extended tag
    - RI:$rfid.last.id

### Fuel
- Added support for serial fuel sensor
- Apx version 22.17.1

  Signals
    Events (Oneshot booleans)
    - @fuel.event                         Triggered when any fuel event is detected.                          
    - @fuel.state                         Triggered when a state arrives.   
    - @fuel.fuelling or @fuel.fueling     Triggered when a fuelling notification arrives. 
    - @fuel.warning                       Triggered when a discharge warining notification arrives.
    - @fuel.connected                     Triggered when a transition from disconnected to connected is detected.
    - @fuel.disconnected                  Triggered when a transition from connected to disconnected is detected.

    States                 
    - $fuel.connected                     Stores the last connected state (boolean).  
    - $fuel.level                         Stores the last fuel level (number).
    - $fuel.temperature                   Stores the last temperature level (number).  
    - $fuel.frequency                     Stores the last frequency (number).
    - $fuel.timestamp                     Stores the last notification timestamp (epoch).
    - $fuel.event                         Stores the last event (string).

  ```syruslang
  # Example of event whenever a card is aproached to the reader
  define fieldset default fields=$fuel
  
  define signal sg_fuel_over $fuel.level > 100
  define signal sg_fuel_under $fuel.level < 10 

  define event ev_fuel group=tracking fieldset=default ack=seq label=over trigger=sg_fuel_over
  define event ev_fuel group=tracking fieldset=default ack=seq label=over trigger=sg_fuel_under
  ```

  TAIP - Extended tag
    - FF added when fields=$fuel

### Bluetooth
  - Fix bluetooth restarting syrusjs bug due a bad json.

### Backlog
  - Fix backlog requesting ack from file  


## v1.34.0

requires `apexOS >= 21.40.2`

### Fatigue

- Updated codebase to use new ECU.d structure
- Support for mdsm7 fatigue signals (prefix with `@fatigue_sensor.` or `@mdsm7.fatigue_sensor.`)
  - camera_blocked
  - distraction
  - fatigue_alarm
  - fatigue_warning
  - phone
  - smoking

```syruslang
# either signal will work
define event fatigue_distraction fieldset=defeault ack=seq group=fatigue label=distract trigger=@mdsm7.fatigue_sensor.distraction
define event fatigue_distraction fieldset=defeault ack=seq group=fatigue label=distract trigger=@fatigue_sensor.distraction
```

### Bluetooth

- BT destination point support `define destination bluetooth_d taip bluetooth://_:_ ack=disabled`
- BT call button signals `@bluetooth.button.call_2x` `@bluetooth.button.signal`
- BT destination points retry indefintely until apx-bt returns succesful (and messages are acked where applicable)

#### Signal, variable and value improvements

`set [variable|value] [name] [value]` infers type (true, false, number, string)

- `set variable new_var "lorem"` or `set variable new_var lorem`
  - {{$variables.new_var}} = “lorem”
- `set variable new_var "lorem ipsum"`
  - {{$variables.new_var}} = “lorem ipsum”
- `set value $gnss.latitude == 25.123456`
- works for any non internal signal (non @ signals)

`fire signal signal_name` does not require @ as prefix

#### TAIP Fixes

- SSL and QXAEC:: do not requiere an ;ID or ;SI when sent remotely to the device
  - >SSLget events<
  - >QXAEC::apx-gps state<
- fixed QSSIP response
- IB/IS syrus3 compatability fix
- AW tag sends empty instead of *

### Other

- http destinations respect url schema
- long commands can be split into multiple lines
- actions can have multiple commands (requires indentation)

### Multiline support

long commands can now be split, indentation is not required but recommended for legibility

```syruslang

define counters globals speed_threshold=100kph
  rpm_threshold=3000rpm
  begin_idle_time=300sec
  odometer=0mts
  ignition_time=0mts

define fieldset default
  fields=$io,$gnss,$net,$ecu,
  VO:$counters.globals.odometer,CE:$counters.globals.ignition_time,
  CL:$counters.globals.idle_time,CS:$counters.globals.over_speed,
  CR:$counters.globals.over_rpm
```

actions can execute multiple commands, any indented commands following `define action` will be assumed to be part of the action.

```syruslang

define action action1 trigger=signal
  set variable test1 1
  set variable test2 2
  set variable test3 3
```

## v1.32.10

- Improved ack support for TAIP/tcp, supported modes are `imei`(default), `disabled` and `seq`
- ECU Errors, SPI, FMI, OC, SM and MIL support
- ECU events supports signals, see the respective pgn in the SDK ECU.d
- MOVON and Mobileye support, loaded from internal json files.

### Bugfixes

- Queues are loaded in proper order
- Improved callbacks for TCP

## v1.31.7

### Fatigue Sensor support

Signals

- @fatigue_sensor.fatigue_remind
- @fatigue_sensor.fatigue_warning
- @fatigue_sensor.fatigue_alarm
- @fatigue_sensor.distraction
- @fatigue_sensor.no_portrait
- @fatigue_sensor.photo

```syruslang
define event fatigue_remind fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.fatigue_remind label=remind trigger=@fatigue_sensor.fatigue_remind
define event fatigue_warning fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.fatigue_warning label=warning trigger=@fatigue_sensor.fatigue_warning
define event fatigue_alarm fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.fatigue_alarm label=alarm trigger=@fatigue_sensor.fatigue_alarm
define event fatigue_distraction fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.distraction label=distract trigger=@fatigue_sensor.distraction
define event fatigue_no_driver fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.no_portrait label=no_driver trigger=@fatigue_sensor.no_portrait
define event fatigue_photo_captured fieldset=defeault ack=seq group=fatigue photo=fatigue_sensor.photo label=photo trigger=@fatigue_sensor.photo
```

### MDT support

- @mdt.message signal is required on the trigger to use the message, it's recommended that you do not mix this with other signals

```syruslang
define event mdt group=mdt fieldset=default code=0 trigger=@mdt.message
```

- TAIP: `STX` is now reserved for MDT, use `SSL` for sending syruslang commands

### Bugfixes

- false|true|Number are parsed as their respective types, not as strings (eg repeated=false)
- min_duration improvements
- improved signal propagation

## v1.30.1

### TPMS support

Signals supported

- @tpms.alarm_ok
- @tpms.sensor_mute
- @tpms.sensor_defective
- @tpms.tire_leak
- @tpms.temperature_warning
- @tpms.over_inflation_critical
- @tpms.over_inflation_warning
- @tpms.under_inflation_warning
- @tpms.under_inflation_critical

## v1.29.0

- Satcom destination support
- TAIP QXASMS, II support (fieldset $satcom)
- improved ack support for TAIP protocol (seq, disabled, imei)
- general tweaks and refactors

### Timers support

```syruslang
define timer timer_name duration=10sec [delay=10sec] [repeat=true] [enabled=true]
# enable|start timer (if not enabled in definition)
start timer timer_name
# stop|disable timer
stop timer timer_name
# use the timer signal in your actions
define action countdown_complete trigger=@timers.timer_name set out2 on
# enable/disable timer from other actions
define action timer_cancel trigger=... stop timer timer_name
```

### IButton

```syruslang
add ibutton john_smith DA00001C3C9C3101
define fieldset taip fields=IB:$ibutton.connected.id,IS:$ibutton.last.id
define fieldset json fields=$ibutton

## SIGNALS
# authorized driver
define signal auth_ibutton $ibutton.connected.whitelisted == true

# unauthorized driver
define signal unauth_ibutton $ibutton.connected.whitelisted == false
# define signal ibutton $ibutton.connected

## EVENTS

# authorized driver
define event authIbutton group=tracking fieldset=taip ack=seq label=ibmtch code=61 trigger=auth_ibutton
# unauthorized driver
define event unauthIbutton group=tracking fieldset=taip ack=seq label=ibnodet code=62 trigger=unauth_ibutton

## ACTIONS
# action to deactivate output2 if driver authorized and speak welcome message
define action auth_driver_deactivate_out trigger=auth_ibutton set out2 off
define action speakActionIbutton trigger=auth_ibutton speak lang=en 'Welcome {{$ibutton.authorized.last.alias}}. Syrus wishes you an excellent trip'
# action to activate output2 if driver is unauthorized
define action unauth_activate_out trigger=unauth_ibutton set out2 on
define action speakActionNoIbutton trigger=unauth_ibutton speak lang=en 'You are not an authorized driver. Please contact an administrator'
```

### Temperature

```syruslang
define group temp
add temperature_sensor temp1 8D01144D07DBAA28
add temperature_sensor temp2 8D01144D07DBAB28
add temperature_sensor temp3 8D01144D07DBAC28

## FIELDSETS
define fieldset taip fields=EA_A:$temp.aliases.temp1.value,EA_B:$temp.aliases.temp2.value,EA_C:$temp.aliases.temp3.value
define fieldset json fields=$temp

## SIGNALS
# temp1 connected
define signal temp1_connected $temp.aliases.temp1.connected
define signal temp2_connected $temp.aliases.temp2.connected
define signal temp3_connected $temp.aliases.temp3.connected
# temperature above 5°C
define signal temp1_above $temp.aliases.temp1.value > 5
define signal temp2_above $temp.aliases.temp2.value > 5
define signal temp3_above $temp.aliases.temp3.value > 5
# temperature below -8°C
define signal temp1_below $temp.aliases.temp1.value < 8
define signal temp2_below $temp.aliases.temp2.value < 8
define signal temp3_below $temp.aliases.temp3.value < 8

## EVENTS
# temp1 Connection
define event temp1Connected group=temp fieldset=taip ack=seq label=temp1det trigger=temp1_connected
define event temp1Disconnected group=temp fieldset=taip ack=seq label=temp1nodet trigger=temp1_connected,not
# temp above
define event tempAbove group=temp fieldset=taip ack=seq label=tempabv code=61 trigger=temp1_above,temp2_above,or,temp3_above,or
# temp below
define event tempBelow group=temp fieldset=taip ack=seq label=tempblw code=62 trigger=temp1_below,temp2_below,or,temp3_below,or
```

### bugfixes

- application/instance name issue
- destination enable/disable

## v1.26.0

- Usage of new queue algorithm that reduce memory footprint and increase maximum stack of events to almost the whole available memory
- fixed value of internal signal $timestamp
- added new signal equivalent to $timestamp for spanish language \$tiempo
example

```syruslang
set value $tiempo # it returns 'Viernes, Diciembre 4, 2020 8:17 PM'
```

## v1.25.0

- added json in ecu for multiple key values for fieldset=$ecu:$ecu, o fieldset=$ecu,
- added new dev commands for get signals, get variables, get actions, get events, get fieldsets, get tracking_resolutions, get variables
get the raw definition as syruslang from configuration separated by `;`
example:
get signals # returns 'define signal out2on $io.out2;define signal out1on $io.out1'
- fixed issue that without a tracking_resolution syrusjs is not updating internal $gnss references, now data is available every second

## v1.24.0

- added mqtt support for syruslang command in subscribe using new param commands_sub
- added mqtt support for syruslang command in publish using new param commands_pub
- fixed json type of key are being always string under certain conditions

## v1.23.0

- added inputs 3,4,5,6 and 7 and ouputs 3 and 4 support for TAIP
- added short circuit detection to outputs support for TAIP
- added support for changing output 3 and output 4 using TAIP serial expander commands

## v1.22.1

- added 2 new internal signal @geofence.enter and @geofence.exit fired in every geofence entered or exited

```syruslang
# geofence.enter and @geofence.exit are internal signal so no need to define a new signal
define event geoIn code=34 label=geoin fieldset=default trigger=@geofence.enter
define event geoOut code=35 label=geoout fieldset=default trigger=@geofence.exit
```

example:

- added 2 new internal signal @geofence.group.enter and @geofence.group.exit fired in every geofence's group entered or exited
- DEV: added new command fire signal to manually trigger signals.

Example:

```syruslang
fire signal IN1ON #(only available from remote console).
```

- DEV: added new command set value to manually setting non internal values.
Example

```syruslang
set $geofences.test.inside true  #(only available from remote console).
```

## v1.22.0

- added compatibility with apex-20.46.2 and new redis instance
- added new flat retain in event for mqtt retain feature
example:
define event trackpoint retain=true code=1 label=trckpnt fieldset=default trigger=@tracking_resolution.mqtt.signal # this event will be retain by the mqtt broker as the last update available

## v1.21.0-beta

- fix inputs/outputs are on true after reset
- added support for configuration.json using json schema
- now it's possible change the configuration file using the json schema included on the app.
- now it's possible change the destinations file using the json schema included on the app.

## v1.20.0-beta

- fix issue double space on destinations
- upgrade syrus4-nodejs to support future versions of ApexOS (>20.44)
- speak alias now support multiple languages speak lang=es,speak lang=en, speak lang=fr, speak without arguments is passed as lang=en
- fixed ecu for TAIP commands
- since apex-20.44.00 syrusjs will stop processing geofences within geofences.syrus.conf file, you should consider import an export geofences from UI and apx-geofences-manager integrated tool, since is not blocking for the app and has much more options.
- added new command get detination_state that returns enable/disable for the connection:
  example

```syruslang
get destination_state peg
# must return `enable`
```

- improve support for HTTP/HTTPS destinations now support authentication, headers, params, method (by default POST), an better error logs.
- added simple schema definition for json schema support

## v1.19.1-beta

- fix issue that broken syrusjs when command depending of units are proccesed

## v1.19.0-beta

Starting with this version units are more explicit for arguments. follow specifications for syruslang new valid units are

1. speed=kph,mph,m/s
2. heading=deg,rad
3. dist=km,mts,mile,ft
4. time=hr,min,sec
5. cycle=rpm,hz
   old units schema will be supported but will emit a warning

- added new argument for actions and event called rate. Avoid fires event/action multiple times in x time defined.
  example:

```syruslang
define tracking_resolution 5sec
# avoids this event fires more than 5 times in 1min, units could be sec,min,hr
define event trckpnt code=1 label=trckpnt rate=5/1min trigger=@tracking_resolution.track.signal
```

- added new command 'send event EVENT_NAME' for forcely fire events within the destinations points
- added new command 'trigger action ACTION_NAME' for forcely trigger an event
- added new commands 'enable destination NAME' and 'disable destination NAME'. using this commands destination will be disconnected and stop sending events and receiving commands, but event will queued for connection will be available again.
- support for arguments subscribe and publish in mqtt destination

```syruslang
# DEPECRATION WARNING
# define destination mqtt_broker json mqtt://test.mosquitto.org:1883 protocol="mqtt" channelRx="dev/{{$modem.imei}}/messagestx" channelTx="dev/{{$modem.imei}}/messagestx"
# REPLACE FOR
define destination mqtt_broker json mqtt://test.mosquitto.org:1883 protocol="mqtt" subscribe="dev/{{$modem.imei}}/messagestx" publish="dev/{{$modem.imei}}/messagestx"
```

## v1.18.0-beta

- support SXAVO command in TAIP
- implement allowed_interface, allowed_ip and allowed_ssid (, for wifi only) to destinations, so destinations can be set to only works in specific enviroments (interface,ip,ssid)
  example:

```syruslang
# this connection will automatic disable outside wifi network and buffer the commands, when wifi is connected again all the messages on the buffer will be sent
define destination mqtt_broker json mqtt://test.mosquitto.org:1883 protocol="mqtt" subscribe="dev/messagestx" publish="dev/messagestx" allowed_interface=wifi
```

- allow use curly brackets to destination config and other commands
  example:

```syruslang
define destination mqtt_broker json mqtt://test.mosquitto.org:1883 protocol="mqtt" subscribe="dev/{{$modem.imei}}/messagestx" publish="dev/{{$modem.imei}}/messagestx"
```

- Implement variables
  example:

```syruslang
define variable my_var "hello"
# you can define actions for change variables or just use as constants
set variable my_var "good bye"
define action aloha trigger=ignSwitch speak '{{$variables.my_var}}, driver'
```

- added new syruslang signal variable $mph, $kph, $speed, $timestamp, those are version more humanized of the $gnss.mph, $gnss.kph, $gnss.speed and $gnss.timestamp
  example:

```syruslang
speak 'today is {{$timestamp}}. The current speed is {{$kph}} kilometers per hour'
```

- supervise configuration.syruslang for changes and restart automatically the app

## v1.17.4-beta

- GPS data update every second, improve signal change detecttion
- added new alias bluetooth for apx-bt commands

## v1.17.3-beta

- Replace underscore (\_) for blank spaces in speak command

## v1.17.2-beta

- fix is inside notification on geofences and groups

## v1.17.1-beta

- added new configuration file geofences.syrus.conf. geofences.syrus.conf is only evaluated when changes are detected; Avoid to add commands different than signals on this file. (recommended move all geofences previously on configuration.syrus.conf to this file.)
- added new command speak alias for 'apx-mdm tts'. Usage: speak 'text to speech test'. only support english, the text must be encapsulated over single quoutes `'`
- added new values @last_geofence.name,@last_geofence.state and @last_geofence.time contains information about last geofence reported for apx-geofences core tool
- added ability to concatenate syrusjs signal variable on some exec commands.
  example:

```syruslang
$TXspeak 'current location.  Latitude: {{$gnss.latitude}}.  Longitude:{{$gnss.longitude}}. Last Geofence reported: {{@last_geofence.name}} {{@last_geofence.state}}'
```

Variable must be inside notation {{ }}, variable is replaced in real time with current value of the syrusLang signal

## v1.16.1-beta

- fix ecu odometer parameter for TAIP
- fix ecu battery parameter for TAIP

## v1.16.0-beta

- geofences are now evaluated using apx-geofences utils from apexOS core
- added support for detecting process exit and clean disconnection from all the destinations point
- added new 38 parameters from ecu
- added new command set @ecu_params_response
- ecu signals and values now are accesible using id instead of aliases name
  example:

```syruslang
# ENGINE TOTAL FUEL USED
# previously $ecu.eng_tot_fuel now is
$ecu.fee9_5-8
```

## v1.15.2-beta

- added counters
- added new command define counters

```syruslang
# only name is mandatory
define counters name speed_threshold=50kph rpm_threshold=3000rpm begin_idle_time=300s odometer=0mts ignition_time=0s idle_time=0s over_speed=0s over_rpm=0 hard_brakes=0 harsh_fwd_acceleration=0
```

- added new command

```syruslang
# only name is mandatory
# require at least one new value to set
set counters name speed_threshold=50kph rpm_threshold=3000rpm begin_idle_time=300s odometer=0mts ignition_time=0s idle_time=0s over_speed=0s over_rpm=0 hard_brakes=0 harsh_fwd_acceleration=0
```

- added new command

```syruslang
stop counters name
```

- added new command

```syruslang
start counters name
```

- added new command

```syruslang
delete counters name
```

- added new values to syruslang signals

```syruslang
# $counters.namespace.odometer
# $counters.namespace.ignition_time
# $counters.namespace.idle_time
# $counters.namespace.over_speed
# $counters.namespace.over_rpm

# example signal
define signal oil_change_required $counters.tecno.odometer > 20000
```

- added support to syruslang fieldset to use counters

```syruslang
# Example

define counters globals speed_threshold=50kph rpm_threshold=3000rpm begin_idle_time=300s odometer=0mts ignition_time=0s idle_time=0s over_speed=0s over_rpm=0 hard_brakes=0 harsh_fwd_acceleration=0

define counters driver speed_threshold=50kph rpm_threshold=3000rpm begin_idle_time=300s odometer=0mts ignition_time=0s idle_time=0s over_speed=0s over_rpm=0 hard_brakes=0 harsh_fwd_acceleration=0

define fieldset default fields=$io,$ecu,VO:$counters.globals.odometer,CE:$counters.globals.ignition_time,CL:$counters.globals.idle_time,CS:$counters.globals.over_speed,CR:$counters.globals.over_rpm,CV00:$counters.driver.odometer,CV01:$counters.driver.ignition_time


define signal driver_connected $net_wifi.ssid == "MOBILE HOTSPOT"
define action trigger=driver_connected start counters driver
define action trigger=driver_connected,not stop counters driver
```

- changed voltage thresholds and 100% to 4000 mV

## v1.14.4-beta

- improve tracking resolution detection arguments
- fix case when only tracking resolution had time with `m`
- added syruslang syruslang conversion for radianes

## v1.14.3-beta

- fixed empty arguments for tracking resolutions

## v1.14.2-beta

- fixed heading on taip events
- group warning when no group detected

## v1.14.0-beta

- changed signals from @tracking_resolution.signal to @tracking_resolution.{namespace}.signal where namespace is the name of the tracking resolution on define tracking_resolution command
- set tracking_resolution command changed to support new configuration of the tracking resolution, ej:

```syruslang
define signal ignOnSignal $io.ign == true
define tracking_resolution pegasus 10m
define action ignOn trigger=ignOnSignal set @tracking_resolution pegasus 120s 25deg 100mts
```

- removed usage of set @tracking_resolution {name} as usage for change active tracking resolution in favor to have multiple tracking resolutions at the same time
